unit OGLPanel;

interface

uses
  Windows, SysUtils, Classes, Controls, ExtCtrls, dglOpenGL;

type

  TViewMode = (vmPerspective,vmOrthogonal);

  TOGLPanel = class(TPanel)
  private
    { Private-Deklarationen }
    DC : HDC;
    RC : HGLRC;

    FColorBits, FStencilBits,
    FZBits, FAccumBits, FAuxBuffers : byte;
    FRCOptions : TRCOptions;
    FViewMode  : TViewMode;

    FFovy : Single;
    FNearPlane : Single;
    FFarPlane : Single;
    FCamera : array[0..2] of Single;
    FLookAt : array[0..2] of Single;
    FUpVector : array[0..2] of Single;

    FOnRender : TNotifyEvent;

    procedure SetupContext();

    procedure setFovy(value : Single);
    procedure setNearPlane(value : Single);
    procedure setFarPlane(value : Single);
    procedure setCamera(index : integer; value : Single);
    function  getCamera(index : integer):Single;
    procedure setLookAt(index : integer; value : Single);
    function  getLookAt(index : integer):Single;
    procedure setUpVector(index : integer; value : Single);
    function  getUpVector(index : integer):Single;
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    constructor Create(AOwner : TComponent);override;
    destructor Destroy();override;

    function Init():Boolean;
    procedure Render();

    procedure Resize();override;
  published
    { Published-Deklarationen }
    property ColorBits   : byte read FColorBits   write FColorBits;
    property StencilBits : byte read FStencilBits write FStencilBits;
    property DepthBits   : byte read FZBits       write FZBits;
    property AccumBits   : byte read FAccumBits   write FAccumBits;
    property AuxBuffers  : byte read FAuxBuffers  write FAuxBuffers;

    property RCOptions   : TRCOptions read FRCOptions write FRCOptions;
    property ViewMode    : TViewMode read FViewMode write FViewMode;

    property OnRender    : TNotifyEvent read FOnRender write FOnRender;

    property Fovy        : Single read FFovy write setFovy;
    property NearPlane   : Single read FNearPlane write setNearPlane;
    property FarPlane    : Single read FFarPlane write setFarPlane;
    property CameraX     : Single index 0 read getCamera write setCamera;
    property CameraY     : Single index 1 read getCamera write setCamera;
    property CameraZ     : Single index 2 read getCamera write setCamera;
    property LookAtX     : Single index 0 read getLookAt write setLookAt;
    property LookAtY     : Single index 1 read getLookAt write setLookAt;
    property LookAtZ     : Single index 2 read getLookAt write setLookAt;
    property UpVectorX   : Single index 0 read getUpVector write setUpVector;
    property UpVectorY   : Single index 1 read getUpVector write setUpVector;
    property UpVectorZ   : Single index 2 read getUpVector write setUpVector;
  end;

procedure Register;

implementation
{$R *.dcr}

procedure Register;
begin
  RegisterComponents('Beispiele', [TOGLPanel]);
end;

procedure TOGLPanel.setFovy(value : Single);
begin
  if (value > 0) and (value < 90) then
    FFovy := value;
end;

procedure TOGLPanel.setNearPlane(value : Single);
begin
  if value < FFarPlane then
    FNearPlane := value;
end;

procedure TOGLPanel.setFarPlane(value : Single);
begin
  if value > FNearPlane then
    FFarPlane := value;
end;

procedure TOGLPanel.setCamera(index : integer; value : Single);
begin
  FCamera[index] := value;
end;

function  TOGLPanel.getCamera(index : integer):Single;
begin
  Result := FCamera[index];
end;

procedure TOGLPanel.setLookAt(index : integer; value : Single);
begin
  FLookAt[index] := value;
end;

function  TOGLPanel.getLookAt(index : integer):Single;
begin
  Result := FLookAt[index];
end;

procedure TOGLPanel.setUpVector(index : integer; value : Single);
begin
  FUpVector[index] := value;
end;

function  TOGLPanel.getUpVector(index : integer):Single;
begin
  Result := FUpVector[index];
end;



constructor TOGLPanel.Create(AOwner : TComponent);
begin
  inherited;
  InitOpenGL();

  if (CsDesigning in self.Componentstate) then begin
    FColorBits := 32;
    FStencilBits := 16;
    FZBits := 16;
    FAccumBits := 16;
    FAuxBuffers := 0;

    FRCOptions := [opDoubleBuffered];

    FViewMode := vmPerspective;

    FFovy := 60;
    FNearPlane := 1;
    FFarPlane  := 10;

    FCamera[0] := 0;
    FCamera[1] := 0;
    FCamera[2] := 10;

    FLookAt[0] := 0;
    FLookAt[1] := 0;
    FLookAt[2] := 0;

    FUpVector[0] := 0;
    FUpVector[1] := 1;
    FUpVector[2] := 0;
  end;
end;

destructor TOGLPanel.Destroy();
begin
  if not (csDesigning in ComponentState) then begin
    wglMakeCurrent(0,0);
    DestroyRenderingContext(RC);
  end;
//  releaseDC(self.Handle,DC);
  inherited;
end;

procedure TOGLPanel.SetupContext();
begin
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  case FViewMode of
    vmPerspective : gluPerspective(FFovy,Width/Height,FNearPlane,FFarPlane);
    vmOrthogonal  : glOrtho(0,Width,0,Height,FNearPlane,FFarPlane);
  end;
  gluLookAt(FCamera[0],FCamera[1],FCamera[2],
            FLookAt[0],FLookAt[1],FLookAt[2],
            FUpVector[0],FUpVector[1],FUpVector[2]);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
end;

function TOGLPanel.Init():Boolean;
begin
  DC := getDC(self.Handle);
  RC := CreateRenderingContext(DC,FRCOptions,FColorBits,FZBits,FStencilBits,FAccumBits,FAuxBuffers,0);
  ActivateRenderingContext(DC,RC);
  Result := true;
end;

procedure TOGLPanel.Render();
begin
  ActivateRenderingContext(DC,RC);
  SetupContext();
  if @FOnRender <> nil then
    FOnRender(self);
  SwapBuffers(DC);
end;

procedure TOGLPanel.Resize();
begin
  inherited;
  if not (csDesigning in self.Componentstate) then begin
    if DC = 0 then
      exit;
    ActivateRenderingContext(DC,RC);
    glViewPort(0,0,self.Width,self.Height);
    SetupContext();
    Render();
  end;
end;

end.
